/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "editor.h"
#include "parenmatcher.h"
#include "conf.h"

// This file has been slightly modified for Xplode

// Constructor
Editor::Editor(QWidget *parent, const char *name) : QTextEdit(parent, name), hasError(false)
{
	// Set defaults
	document()->setFormatter(new QTextFormatterBreakInWords);
	setHScrollBarMode(QScrollView::AlwaysOff);
	setVScrollBarMode(QScrollView::AlwaysOn);
	document()->setUseFormatCollection(false);
	parenMatcher = new ParenMatcher;
	// Connect signals
	connect(this, SIGNAL(cursorPositionChanged(QTextCursor *)), this, SLOT(cursorPosChanged(QTextCursor*)));
	cfg = new Config;
	// Set default styles
	document()->addSelection(Error);
	document()->addSelection(Step);
	document()->setSelectionColor(Error, red);
	document()->setSelectionColor(Step, yellow);
	document()->setInvertSelectionText(Error, false);
	document()->setInvertSelectionText(Step, false);
	document()->addSelection(ParenMatcher::Match);
	document()->addSelection(ParenMatcher::Mismatch);
	document()->setSelectionColor(ParenMatcher::Match, QColor(204, 232, 195));
	document()->setSelectionColor(ParenMatcher::Mismatch, Qt::magenta);
	document()->setInvertSelectionText(ParenMatcher::Match, false);
	document()->setInvertSelectionText(ParenMatcher::Mismatch, false);
}

// Handle cursor position changes
void Editor::cursorPosChanged(QTextCursor *c)
{
	if (parenMatcher->match(c)) repaintChanged();
	if (hasError)
	{
		emit clearErrorMarker();
		hasError = false;
	}
}

// If configuration changes, repaint the viewport
void Editor::configChanged()
{
	document()->invalidate();
	viewport()->repaint(false);
}

// Overloaded QTextEdit function
void Editor::doChangeInterval()
{
	emit intervalChanged();
	QTextEdit::doChangeInterval();
}

// Overloaded QTextEdit function
bool Editor::eventFilter(QObject *o, QEvent *e)
{
	return QTextEdit::eventFilter(o, e);
}
