#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfile.h>
#include <qdom.h>
#include <qxml.h>
#include <qprogressdialog.h>

#include "plasmfunctionlistbox.h"
#include "plasmfunctionlistitem.h"
#include "plasmkeywords.h"
#include "xplodeshared.h"
#include "xplodeicons.h"

extern QProgressDialog *initDialog;

extern PlasmKeywords *plasmKeywords;

PlasmFunctionListBox::PlasmFunctionListBox(QWidget *parent, const char *name) : QListView(parent, name)
{
        addColumn("Name", 150);
        addColumn("Input", 100);
        addColumn("Output", 100);
        addColumn("Description", 500);
        setRootIsDecorated(true);
        setSelectionMode(QListView::Single);
        setResizeMode(QListView::NoColumn);
        setVScrollBarMode(QScrollView::Auto);
        setHScrollBarMode(QScrollView::Auto);

        // The ``Alphabetical'' item is always present
        PlasmFunctionListItem *alpha = new PlasmFunctionListItem(this, "Alphabetical");
        PlasmFunctionListItem *search, *item;
        int progress = 1;
        initDialog->setLabelText("Parsing keywords...");
        initDialog->setTotalSteps(plasmKeywords->db.size() + 1);
        initDialog->setProgress(progress);
        // Cycle in all definitions and update the tree
        QMap<QString, KeyWord*>::Iterator it;
        for (it = plasmKeywords->db.begin(); it != plasmKeywords->db.end(); ++it)
        {
                // Add the item to the alphabetical list
                item = new PlasmFunctionListItem(alpha, it.data()->getName());
                // Does the library it belongs to exist?
                search = (PlasmFunctionListItem*) findItem(it.data()->getLibrary(), 0);
                // If the library doesn't exist, create it
                if (search == 0)
                {
                        // New library item
                        search = new PlasmFunctionListItem(this, it.data()->getLibrary());
                }
                item = new PlasmFunctionListItem(search, it.data()->getName());
                item->setDefinitionName(it.data()->getName());
                item->setInput(it.data()->getInput());
                item->setOutput(it.data()->getOutput());
                item->setDescription(it.data()->getDescription());
                progress++;
                initDialog->setProgress(progress);
        }
}


void PlasmFunctionListBox::contentsMousePressEvent(QMouseEvent *event)
{
        // Store the drag position
        if (event->button() == LeftButton)
                dragPos = event->pos();
        QListView::contentsMousePressEvent(event);
}

void PlasmFunctionListBox::contentsMouseMoveEvent(QMouseEvent *event)
{
        // Are we still pressing the left button? Are we NOT selecting a top level item?
        if (selectedItem() != 0)
        {
                if ((event->state() & LeftButton) && (selectedItem()->childCount() == 0))
                        startDrag();
        }
        QListView::contentsMouseMoveEvent(event);
}

void PlasmFunctionListBox::startDrag()
{
        // Get the selected text
        QString text = currentItem()->text(0);
        if (!text.isEmpty())
        {
                // Create the drag item
                QTextDrag *dragText = new QTextDrag(text, this);
                // Cool pixmap
                dragText->setPixmap(QPixmap(qembed_findImage("plasmevalexpr")));
                dragText->dragCopy();
        }
}

void PlasmFunctionListBox::contentsDragEnterEvent (QDragEnterEvent *event)
{
        if (event->source() == this)
                event->ignore();
        else
                event->accept();
}

void PlasmFunctionListBox::contentsDragMoveEvent (QDragMoveEvent *event)
{
        if (event->source() == this)
                event->ignore();
        else
                event->accept();
}

void PlasmFunctionListBox::contentsDropEvent (QDropEvent *event)
{
        if (event->source() == this)
                event->ignore();
        else
        {
                QString text;
                event->accept();
        }
}
