#include "xplodedoc.h"
#include "xplodeshared.h"

// Constructor
XplodeDoc::XplodeDoc()
{
	modified = false;
}

// Destructor
XplodeDoc::~XplodeDoc()
{
}

// Connect the document to its editor
void XplodeDoc::connectEditor(XplodeEditor *editor)
{
	docEditor = editor;
	// Connect the signal editor contents have been modified
	connect(docEditor, SIGNAL(textChanged()), this, SLOT(setModified()));
	connect(docEditor, SIGNAL(s_NewCursorPos(int, int)), this, SLOT(cursorPosition(int, int)));
}

// Set the document complete path name
void XplodeDoc::setPathName(QString name)
{
	docFileName = name;
	docTitle = QFileInfo(name).fileName();
}

// Return the document complete path name
QString XplodeDoc::pathName()
{
	return docFileName;
}

// Set the document title
void XplodeDoc::setTitle(QString title)
{
	docTitle = title;
}

// Return the document title
QString XplodeDoc::title()
{
	return docTitle;
}

// Create an empty document
bool XplodeDoc::newDocument()
{
	modified = false;
	return true;
}

// Open an existing document
bool XplodeDoc::openDocument(QString filename)
{
	QFile f(filename);
	// Open the file, on error exit
	if (!f.open(IO_ReadOnly)) return false;
	// Open the file as text
	QTextStream file(&f);
	// Append the text to the editor
	docEditor->setText(file.read());
/*	int para = 0;
	while (!file.eof())
	{
		docEditor->insertParagraph(file.readLine(), para);
		para++;
	}*/
	// Close the file
	f.close();
//	docEditor->repaint();
	// Set defaults (title, path, modified)
	modified = false;
	docFileName = filename;
	docTitle = QFileInfo(f).fileName();
	// Well done
	return true;
}

// Save the document
bool XplodeDoc::saveDocument(QString filename)
{
	// Fix the suffix
	if (!filename.lower().endsWith(".psm")) filename.append(".psm");
	QFile f(filename);
	// Open file for writing, on error exit
	if (!f.open(IO_WriteOnly)) return false;
	// Open file as pure text
	QTextStream file(&f);
	// Get the text and save it
	file << docEditor->text();
	// Close file
	f.close();
	// Set defaults (title, path, modified)
	modified = false;
	docFileName = filename;
	docTitle = QFileInfo(f).fileName();
	emit documentSaved(this);
	// Well done
	return true;
}

// Delete document content
void XplodeDoc::deleteContents()
{
	// Clear the document editor
	docEditor->clear();
}

bool XplodeDoc::canCloseDocument()
{
	// If the document is unmodified and named, return true
	if (!isModified() && !title().contains("Untitled")) return true;
	// If the document is modified
	if (isModified())
	{
		QString saveName;
		switch (QMessageBox::information(0, "Xplode - " + title(), "The current document has been modified.\n"
				"Do you want to save it?", QMessageBox::No, QMessageBox::Yes, QMessageBox::Cancel | QMessageBox::Default))
		{
			// Save the document
			case QMessageBox::Yes:
			{
				// If the document is unnamed
				if (title().contains("Untitled"))
				{
					// Get a document name
					saveName = QFileDialog::getSaveFileName(0, "Plasm documents (*.psm);;All files (*)", 0);
					// If saveName is invalid, exit
					if (saveName.isEmpty()) return false;
				}
				else
					saveName = pathName();
				// An error occurred saving
				if (!saveDocument(saveName))
				{
					switch (QMessageBox::critical(0, "Xplode - Error", "Could not save the current document,\n"
							"Close anyway ?", QMessageBox::Yes, QMessageBox::No | QMessageBox::Default))
					{
						case QMessageBox::Yes: return true;
						case QMessageBox::No:  return false;
					}
				}
				// All things gone
				else
					return true;
				break;
			}
			case QMessageBox::No: return true;
			case QMessageBox::Cancel:
			default: return false;
		}
	}
	else return true;
	// This should never be reached
	return true;
}

void XplodeDoc::setModified(bool m)
{
	// Emit the signal if the state changes from not modified to modified
	if (!modified && m) emit documentModified(this);
	modified = m;
}

void XplodeDoc::cursorPosition(int para, int idx)
{
	QString msg = "L: %1, C: %2";
	emit documentCursor(msg.arg(para + 1).arg(idx));
}
